% Clear all
clear all
close all
clc

% Add path for the API scripts
addpath('../API')


%%
vrep=remApi('remoteApi');      % Creates remote API object which is called vrep
vrep.simxFinish(-1);           % Closes any currently open connections
 
clientID = vrep.simxStart('127.0.0.1',19999,true,true,5000,5);   % Creates a connection
 
if (clientID>-1)
    disp('Connected')
    
    % Load the robot data
    load('KUKA_LBR_iiwa_7_R800.mat');
    
    % Get object handles
     n = length(axisOfRotation(:,1));
    jointHandles = zeros(n,1);
    for i=1:1:n
        stringJoint = sprintf('LBR_iiwa_7_R800_joint%i',i);
        [returnCode,jointHandles(i)] = vrep.simxGetObjectHandle(clientID,stringJoint,vrep.simx_opmode_blocking);
        if returnCode < 0
            error('Can not find all joint handles!');
        end
    end
    [returnCode,joint1Position] = vrep.simxGetObjectPosition(clientID,jointHandles(1),-1,vrep.simx_opmode_blocking);
    
    % Start with all thetas equal to zero
    theta = zeros(n,1);
    p_d = [0.7; 0.2; 0.4];
    e = p_d - getEndposition(axisOfRotation,theta,lengthVectors,joint1Position,false);
    alpha = 0.0001;
    treshold = 1*10^(-2);

    while (norm(e) > treshold)
        J = vrepJacobianEndposition(axisOfRotation,theta,lengthVectors,joint1Position);
        J_inv = pinv(J);
        dTheta = alpha*J_inv*e;
        theta = theta + dTheta;
        e = p_d - getEndposition(axisOfRotation,theta,lengthVectors,joint1Position,false);
        norm(e)
    end
    
    % Set the target position of a joint, therefore the control loop has to
    % be enabled
    for i=1:1:n
        [returnCode] = vrep.simxSetJointTargetPosition(clientID,jointHandles(i),theta(i),vrep.simx_opmode_blocking);
        if returnCode < 0
            error('Can not set joint target positions!');
        end
    end
    disp('Calculated Endposition')
    getEndposition(axisOfRotation,theta,lengthVectors,joint1Position,true)
        
    vrep.simxFinish(-1); 
end
 
vrep.delete();            % Destructor